# Copyright (C) 2008 Canonical, Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import posixpath
import re
import time
from urllib2 import urlopen, HTTPError
from xmlrpclib import Transport, ServerProxy, Fault

# Assumes the tracd is already running and pointed
# at a token server that allows decafbad.
TRAC_URL = 'http://localhost:8001'


class LoginFailed(Exception): pass

def launchpad_login(server, token):
    url = posixpath.join(server, "launchpad-auth", token)
    try:
        r = urlopen(url)
    except HTTPError, e:
        raise LoginFailed(str(e))
    cookie = r.headers['Set-Cookie']

    m = re.match("(trac_auth=[a-zA-Z0-9]+)", cookie)
    if not m:
        raise LoginFailed("Auth cookie not provided")
    return m.group(1)


class CookieTransport(Transport):
    """xmlrpclib transport subclass to support providing a cookie with every
    request"""

    def __init__(self, cookie):
        self.cookie = cookie
        Transport.__init__(self)

    def send_host(self, connection, host):
        connection.putheader("Host", host)
        connection.putheader("Cookie", self.cookie)

trans = CookieTransport(launchpad_login(TRAC_URL, "decafbad"))
server = ServerProxy(posixpath.join(TRAC_URL, "xmlrpc"), transport=trans)

print 'Server is running:', server.launchpad.bugtracker_version()

