"""
For now this server always allows certain tokens, showing which ones
are used to stdout.  Eventually it should be modified to explicitly
only allow a token to be used once.
"""

# Copyright (C) 2008 Canonical, Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import re
import sys

import BaseHTTPServer

OK_MSG = """
<html><p>Valid token</p></html>
"""

BAD_MSG = """
<html><p style="color: red">INVALID</p></html>
"""

def runserver(allow, deny, default_allow):
    """Specify whether tokens should be allowed or denied."""

    class Handler(BaseHTTPServer.BaseHTTPRequestHandler):
        def do_POST(self):
            m = re.match(r'^/token/([a-zA-Z0-9]+)/\+bugtracker-handshake$', self.path)
            if not m:
                self.send_response(404)
                return
            token = m.group(1)
            r = default_allow
            if token in allow:
                r = True
            elif token in deny:
                r = False
            print "Returning token for", repr(token), "allow =", r
            self.send_response(r and 200 or 404)
            self.end_headers()
            self.wfile.write(r and OK_MSG or BAD_MSG)
    s = BaseHTTPServer.HTTPServer(('localhost', 8081), Handler)
    s.serve_forever()

if __name__ == '__main__':
    allow = []
    deny = []
    default = None
    for c in sys.argv[1:]:
        if c == '+':
            default = True
        elif c == '-':
            default = False
        elif c.startswith('+'):
            allow.append(c[1:])
        elif c.startswith('-'):
            deny.append(c[1:])
        else:
            print "Unknown spec", c
    if default is None:
        print "Usage: +allow1 +allow2 -deny1 -deny2 [+|-]"
        print "The lone +/- specifies the default policy"
        sys.exit(1)
    print "Allowing:", allow
    print "Denying:", deny
    print "Default policy:", (default and "allow" or "deny")
    runserver(allow, deny, default)
